#include "stdafx.h"
#include "testmemmap.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BEGIN_MESSAGE_MAP(CTestmemmapApp, CWinApp)
	//{{AFX_MSG_MAP(CTestmemmapApp)
	//}}AFX_MSG
END_MESSAGE_MAP()

CTestmemmapApp::CTestmemmapApp()
{
}

CTestmemmapApp theApp;

BOOL CTestmemmapApp::InitInstance()
{
  //First test out the class when we want to map a file
  //You might want to step through this code to test it out
  CMemMapFile mmf;
  BOOL bSuccess = mmf.MapFile(_T("C:\\CONFIG.SYS"), FALSE);
  LPVOID pData = mmf.Open();
  BOOL bIsOpen = mmf.IsOpen();
  CString sName = mmf.GetMappingName();
  bSuccess = mmf.Flush();
  #ifdef _DEBUG
  afxDump << mmf;
  #endif
  bSuccess = mmf.Close();
  bIsOpen = mmf.IsOpen();
  sName = mmf.GetMappingName();
  mmf.UnMap();

  //As above but with read only access, Writing into the 
  //returned pointer should then cause an access violation
  //hence the need for exception handling code
  CMemMapFile mmf2;
  bSuccess = mmf2.MapFile(_T("C:\\CONFIG.SYS"), TRUE);
  pData = mmf2.Open();
  try
  {
    ((BYTE*) pData)[0] = 3;
  }
  catch (...)
  {
    TRACE("Unhandled exception caught\n");
  }
  bIsOpen = mmf2.IsOpen();
  sName = mmf2.GetMappingName();
  bSuccess = mmf2.Flush();
  #ifdef _DEBUG
  afxDump << mmf2;
  #endif
  bSuccess = mmf2.Close();
  bIsOpen = mmf2.IsOpen();
  sName = mmf2.GetMappingName();
  mmf2.UnMap();


  //Now a real demonstration of Memory mapped file usage
  //Backup the file CONFIG.SYS to CONFIG.PJ and make it all
  //uppercase. Error checking is ignored for demonstration purposes
  //The real power of memory mapped files is that this demonstration
  //would work just as efficiently with a 100 MB file as it does with
  //just a small file like config.sys.
  CopyFile(_T("C:\\CONFIG.SYS"), _T("C:\\CONFIG.PJ"), TRUE);
  CMemMapFile mmf3;
  mmf3.MapFile(_T("C:\\CONFIG.PJ"), FALSE);
  pData = mmf3.Open();
  CharUpperBuff((char*) pData, mmf3.GetLength());
  mmf3.UnMap();
  AfxMessageBox(_T("The file c:\\config.pj should be the uppercase version of c:\\config.sys"));


  //Finally a demonstration of the shared memory capabilities of 
  //memory mapped files. For this demo, we will bring up a UI which
  //will take whatever is put into its edit box and transfer it into 
  //shared memory. Then if you bring up another version of testmemmap, 
  //you will see this value being reflecting into it.
  //periodically
  CDialog1 dlg;
  dlg.DoModal();

	return FALSE;
}


CDialog1::CDialog1(CWnd* pParent /*=NULL*/)
	: CDialog(CDialog1::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDialog1)
	//}}AFX_DATA_INIT
}


void CDialog1::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDialog1)
	DDX_Control(pDX, IDC_EDIT1, m_ctrlEdit);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDialog1, CDialog)
	//{{AFX_MSG_MAP(CDialog1)
	ON_EN_CHANGE(IDC_EDIT1, OnChangeEdit1)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


BOOL CDialog1::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
  //Check to see if we are the first instance
  //of this program running. In the process testing
  //the MapExistingMemory method
	if (m_mmf.MapExistingMemory(_T("MMFTEST"), 20*sizeof(TCHAR)))
    AfxMessageBox(_T("This program is already running"));
  else
    AfxMessageBox(_T("This program is the first instance"));
  m_mmf.UnMap();

  //map the shared memory, 
	if (!m_mmf.MapMemory(_T("MMFTEST"), 20*sizeof(TCHAR)))
    AfxMessageBox(_T("Failed to map shared memory"));

  m_ctrlEdit.LimitText(20);
	
	return TRUE;
}

void CDialog1::OnChangeEdit1() 
{
	TCHAR sText[20];
  m_ctrlEdit.GetWindowText(sText, 20);
  LPVOID lpData = m_mmf.Open();
  if (lpData)
    _tcsncpy((TCHAR*)lpData, sText, 20);
  m_mmf.Close();
}

int CDialog1::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CDialog::OnCreate(lpCreateStruct) == -1)
		return -1;
	
  //Create a 1 second timer to allow us to update the
  //edit box
	m_nTimer = SetTimer(1, 1000, NULL);
	
	return 0;
}

void CDialog1::OnDestroy() 
{
  KillTimer(m_nTimer);

	CDialog::OnDestroy();
}

void CDialog1::OnTimer(UINT /*nIDEvent*/) 
{
  //Retreive the contents of the MMF and
  //update the edit control
	TCHAR sText[20];
	LPVOID lpData = m_mmf.Open();
  if (lpData)
    _tcsncpy(sText, (TCHAR*) lpData, 20);
  m_mmf.Close();
  m_ctrlEdit.SetWindowText(sText);
}
